#if !defined( _XServer_h )
#define _XServer_h

#include "JAddObj.h"
#include "JSocket.h"
#include "JServerSocket.h"
#include "JSocketInputStream.h"
#include "JSocketOutputStream.h"
#include "JThread.h"
#include "JCriticalSection.h"

#define MaxValues 100
#define InMaxValues 10
#define MaxTemplateVars 100

class XServer : public JAddObj {
  protected:
    virtual void writeContent(class JOutputStream& os);
    virtual void readContent(class JDictionary& dict);
  public:
    enum {
      READINPUT, TEMPLATETEXT,
		  INT_0, INT_1, INT_2, INT_3, INT_4, INT_5, INT_6, INT_7, INT_8, INT_9,
	REAL_0, REAL_1, REAL_2, REAL_3, REAL_4, REAL_5, REAL_6, REAL_7, REAL_8, REAL_9,
	STRING_0, STRING_1, STRING_2, STRING_3, STRING_4, STRING_5, STRING_6, STRING_7, STRING_8, STRING_9 };

    virtual const char* className() const;
    virtual JObject* clone() const;
    XServer();
    XServer(const XServer& obj);
    virtual ~XServer();
    virtual void reset();
    virtual void open( int openPort );
    virtual void close();
    virtual void openSocket();
    virtual boolean connectSocket();
    virtual void closeSocket(int n);
    virtual int inputType(int n);
    virtual boolean inputAllowed(int n);
    virtual JString inputTag(int n);
    virtual JString outputTag(int n);
    virtual JFRect inputArea(int n);
    virtual JFRect outputArea(int n);
	virtual int outputType(int n);
    virtual int inputFace(int n);
	virtual int outputFace(int n);
    virtual JFPoint inputPoint(int n, JLinkObj& link);
    virtual JFPoint outputPoint(int n, JLinkObj& link);
    virtual void paint(JGraphics g, double dx, double dy, JRegion& rgn, double scale);
    virtual void draw(JGraphics g, int x, int y, int w, int h);
    virtual void engine(int n, JLinkObj& link);
    virtual void run();
    virtual JArray getProperties();
    virtual boolean updateProperty(JProperty& prop);
    virtual void access(int n, JLinkObj& link, const JDataType& data);
    int processInputStream();
  protected:
	  void initData();
	  int processResult( char* b, int size, int start );
	int tempVal;

	void writeData();
	void setOutputs();  // set output values based on requestedURL
	void setIntOutput( char* scanner );
	void setRealOutput( char* scanner );
	void setStringOutput( char* scanner );
	bool madeConnection;
    JSocket socket;
	JServerSocket ssocket;
    JSocketInputStream sis;
    JSocketOutputStream sos;

    boolean connected;
	boolean wroteData;
    JThread thread;
    JArray data;
    JCriticalSection cs;
	int outIdx;
	int workingOnValue;

    // controls
	JString templateText;
    int readInput;

	// inputs
	int intIn[InMaxValues];
	double floatIn[InMaxValues];
	JString stringIn[InMaxValues];

    // properties
    int port;
	bool debug;
	bool disableBrowserCaching;
	bool synchOutputsWithInputs;
	int numberOutputValues;
	int htmlRefreshInterval;


	// since we can open one or other, need to track current
	boolean connecting;   // delays thread until other requests are finished...
	int currentPort;
	boolean xprocessResult;

    // outputs
    int intOut[MaxValues];
	double floatOut[MaxValues];
	JString stringOut[MaxValues];
	JString requestedURL;

	// variables used to create server output
	enum { XML_CONTENT, HTML_CONTENT };
	int contentType;
	enum { NO_MAPPER, INT_MAPPER, REAL_MAPPER, STRING_MAPPER };
	char sprintf_format[2000];
	char serverOutput[2000];
	char sprintf_values[ MaxTemplateVars ][50];
	int mapperTypes[ MaxTemplateVars ];
	int mapperOffsets[ MaxTemplateVars ];

	// private methods for creating server output
	void createMapper( int* mapperType, int* mapperOffset, char* templateIndicator );
	void mapValue( int mapperType, int mapperOffset, char* sprintf_value );
	void createSprintfFormat();
	void createServerOutput();
};

#endif
